package org.xbib.multiserv.plugin.printrlserv;

import java.util.ArrayList;
import org.xbib.multiserv.api.Plugin;
import org.xbib.multiserv.api.PluginException;
import org.xbib.multiserv.api.Command;
import org.xbib.multiserv.api.CommandFunction;
import org.xbib.multiserv.api.CommandServer;
import org.xbib.multiserv.plugin.printrlserv.util.HttpClientHelper;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

public class PrintRLServ implements Plugin {

    private static final Logger logger = Logger.getLogger(PrintRLServ.class.getName());

    private HttpClientHelper httpClientHelper;

    private String url;

    public PrintRLServ() {
        super();
    }

    @Override
    public void onStart() throws IOException {
        this.httpClientHelper = new HttpClientHelper();
        this.url = System.getProperty("multiserv.printrl.url", "http://localhost:8010/docs/printrl");
        logger.log(Level.INFO, "PrintRLServ up, forwarding to " + url);
    }

    @Override
    public List<CommandFunction> getFunctions(CommandServer commandServer) {
        List<CommandFunction> list = new ArrayList<>();
        list.add(CommandFunction.of("SLNPFLBestellung", this, SLNPFLBestellung.class));
        return list;
    }

    @Override
    public void onExecute(CommandServer commandServer,
                          Command<?> command,
                          Map<String, Object> params,
                          List<String> response) throws PluginException {
    }

    public class SLNPFLBestellung extends Command<CommandServer> {

        public SLNPFLBestellung() {
        }

        @Override
        public List<String> execute(Map<String, Object> params) {
            if (params.containsKey("SpecIssue")) {
                String specIssue = (String) params.get("SpecIssue");
                if ("RLS".equals(specIssue)) {
                    logger.log(Level.INFO, "Leihscheindruck");
                    try {
                        httpClientHelper.post(url,
                                httpHeaders -> {
                                },
                                parameter -> {
                                },
                                createBody(params),
                                httpResponse -> {
                                },
                                errorResponse -> {
                                },
                                timeOut -> {
                                }
                        );
                        return Arrays.asList(
                                "600 SLNPFLBestellung",
                                "601 OKMsg: Accepted",
                                "250 SLNPEndOfData",
                                "");
                    } catch (IOException e) {
                        logger.log(Level.SEVERE, e.getMessage(), e);
                        return Arrays.asList(
                                "600 SLNPFLBestellung",
                                "510 NotOK: " + e.getMessage(),
                                "250 SLNPEndOfData",
                                "");
                    }
                }
            }
            return List.of();
        }
    }

    @Override
    public void onShutdown() {
    }

    private String createBody(Map<String, Object> parameters) {
        StringBuilder sb = new StringBuilder();
        parameters.forEach((k, v) -> {
            sb.append(k).append(':');
            if (v != null) {
                sb.append(v);
            }
            sb.append('\n');
        });
        return sb.toString();
    }
}
